==> Immersitech-ClearVoice-SDK-v2.0.0-MacOS.tar.gz: Immersitech-ClearVoice-SDK-v2.0.0-MacOS/include/immersitech_general.h <==
#pragma once

#ifdef HAVE_VISIBILITY
#if HAVE_VISIBILITY
#define IMMERSITECH_LOGGER_API __attribute__((__visibility__("default")))
#else
#define IMMERSITECH_LOGGER_API
#endif
#elif defined _WIN32
#define IMMERSITECH_LOGGER_API __declspec(dllexport)
#else
#define IMMERSITECH_LOGGER_API
#endif

#ifdef __ANDROID__
#include <android/log.h>
#define  LOG_TAG    "immersitech"
#define  LOGD(...)  __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__)
#endif

#include <string>
#include <mutex>

/// @brief All error codes that may be produced by the library
///
typedef enum imm_error_code {
	IMM_ERROR_START = -10001,				///< This value allows you to identify the start of the enum
	IMM_ERROR_NONE,							///< This error code indicates the function worked successfully without error
	IMM_ERROR_HANDLE_NULL,					///< This error code indicates you have not initialized your imm_handle properly yet.
	IMM_ERROR_DATA_NULL,					///< This error code indicates you tried to use a buffer or variable that wasn't allocated properly.
	IMM_ERROR_DATA_LENGTH,					///< This error code indicates there is a mismatch between the size of the data buffer you provided and the size required.
	IMM_ERROR_NUM_CHANNELS,					///< This error code indicates that you tried to specify a number of channels outside the allowed bounds. Currently only 1 or 2 channels are allowed.
	IMM_ERROR_SAMPLE_RATE,					///< This error code indicates that you tried to specify an output sampling rate that is unsupported or an input sampling rate that doesn't work with the output sampling rate.
	IMM_ERROR_SPATIAL_QUALITY,				///< This error code indicates that you specified an invalid spatial quality value. Allowable values are from 1 to 5.
	IMM_ERROR_INVALID_ROOM_ID,				///< This error code indicates that you requested a room with an ID that doesn't exist or tried to create a room with an ID that already existed.
	IMM_ERROR_INVALID_PARTICIPANT_ID,		///< This error code indicates that you requested a participant with an ID that doesn't exist or tried to add a participant with an ID that already existed.
	IMM_ERROR_INVALID_CONTROL,				///< This error code indicates the control specified is not a valid one with the bounds of imm_audio_control
	IMM_ERROR_INVALID_VALUE,				///< This error code indicates the value provided is outside the bounds specified for the various imm_audio_control. It can also indicate you provided an invalid function parameter in general.
	IMM_ERROR_INVALID_HEADING,				///< This error code indicates you provide a heading that was outside the allowed values. Please reference imm_heading for allowed values.
	IMM_ERROR_INVALID_JSON_FILE,            ///< This error code indicates a syntax error in the provided JSON file (parameter, formating).
	IMM_ERROR_NO_INPUT_AUDIO,				///< This error code indicates that there was no input audio to be processed, meaning an output buffer could not be generated. Instead the output buffer will be zeroed.
	IMM_ERROR_PARTICIPANT_TYPE,				///< This error code matches the exceptions specified under imm_participant_type.
	IMM_ERROR_LIBRARY_NOT_YET_INITIALIZED,	///< This error code will be generated if you call any functions before initializing the library.
	IMM_ERROR_LICENSE_DATE_EXPIRED,			///< This error code indicates your license files no longer matches the date requirements. All audio effects will be bypassed.
	IMM_ERROR_LICENSE_VERSION_MISMATCH,		///< This error code indicates your license files version requirements no longer match. All audio effects will be bypassed.
	IMM_ERROR_LICENSE_TAMPERED,				///< This error code indicates your license file has been corrupted. All audio effects will be bypassed.
	IMM_ERROR_LICENSE_MISSING,				///< This error code indicates the path to your license file is invalid. All audio effects will be bypassed.
	IMM_ERROR_FILE_DOES_NOT_EXIST,          ///< This error code indicates that an invalid file path was provided (invalid path or invalid filename).
	IMM_ERROR_MISSING_OR_INVALID_PARAMETER, ///< This error code indicates that a parameter is either missing or invalid.
	IMM_ERROR_INVALID_FUNCTION_CALL,        ///< This error code indicates you called a function that is incompatible with your configuration, for example calling imm_cv_add_stream() while multistream_mode is disabled.
    IMM_ERROR_INVALID_STREAM_ID,            ///< This error code indicates that you supplied an invalid stream ID. Perhaps you've already removed this stream?
    IMM_ERROR_END							///< This value allows you to identify the end of the enum
}imm_error_code;

/// @brief Log level definition.
///
/// A message written to the log can have one of these levels of severity
typedef enum {
	IMM_LOG_VERBOSE,	///< This level is used for anything, no matter how unimportant
	IMM_LOG_DEBUG,		///< This level should be used only during a debugging session. It should print information that is helpful to the debugging process 
	IMM_LOG_INFO,		///< This level can be used to print generic information. For example, a participant was added to a room
	IMM_LOG_WARNING,	///< This level should be used for something that happened that could lead to an error or unexpected behavior
	IMM_LOG_ERROR		///< This level is exclusively for critical errors that have affected functionality
} imm_log_level;

/// @brief Function to return the current version of the immersitech library
///
/// @return A string containing the version information for this instance of the Immersitech Library
extern "C" IMMERSITECH_LOGGER_API const char* imm_get_version();

/// @brief Function to turn on or off logging within the Immersitech Library
/// 
/// If you disable logging by sending false to this function, nothing will ever be logged. No messages will be printed.
/// If you enable logging by sending true to this function. The Immersitech Library will log all messages equal to or above the set logging level.
/// By default, Immersitech Library will log all messages to the standard output.
/// By default, Immersitech Library will set the log level to Warnings and above only.
/// @param[in] enable This bool dictates whether you'd like to enable logging or not
/// @return An imm_error_code value specifying whether or not the call was a success
extern "C" IMMERSITECH_LOGGER_API imm_error_code imm_enable_logging(bool enable);

/// @brief Function to set the log level of the Immersitech logger
/// 
/// The logger will only log messages that are equal to or greater than the currently set log level.
/// @param[in] level The log level you'd like to set. check the imm_log_level enum for details on which integer value corresponds to which log level
/// @return An imm_error_code value specifying whether or not the call was a success
extern "C" IMMERSITECH_LOGGER_API imm_error_code imm_set_log_level(int level);

/// @brief Get a human readable string for an error code
///
/// This function allows you to print an error code enum value as its human readable equivalent by returning a string
/// @param[in] error_code The error code you'd like to get human readable form for
/// @return A string representing the corresponding error code enum value
extern "C" IMMERSITECH_LOGGER_API const char* imm_error_code_to_string(imm_error_code error_code);

/// @brief Default Immersitech logger handler
///
/// If you would like to expand on the logging system, you can inhert this class to implement your own behavior specific to your system.
class IMMERSITECH_LOGGER_API imm_logger_handler {
public:
	/// @brief Construct a new immersitech logger handler object
	///
	imm_logger_handler();

	/// @brief Destroy the immersitech logger handler object
	///
	virtual ~imm_logger_handler();

	/// @brief This function will process a message and post it to the appropriate log
	///
	/// If you inherit this class to make a custom logger, this is the function that will handle most of the processing.
	/// @param[in] level The log level which this message should be printed with
	/// @param[in] str The message that is to be logged
	virtual void handle(imm_log_level level, const char* str);
};

/// @brief Logger class.
///
class IMMERSITECH_LOGGER_API imm_logger {
	/// @brief The pointer to the logging instance
	///
	static imm_logger* m_pThis;
public:

	/// @brief Destroy the logger object
	///
	virtual ~imm_logger();

	/// @brief Initialize logger with speficied handler.
	///
	void initialize(imm_logger_handler* handler);

	/// @brief Get the logger object
	///
	/// @return A pointer to the logger
	static imm_logger* get_logger();

	/// @brief write a message to a log
	///
	/// @param[in] level The log level which this message should be printed with 
	/// @param[in] format A character array that can be overload just like the printf() function. For example: log(IMM_LOG_DEBUG, "The number is %i for participant %s", number, name);
	virtual void log(imm_log_level level, const char* format, ...);

	/// @brief Convert log level to string.
	///
	/// @param[in] level log level as an enum value
	/// @return log leve as a string
	static std::string to_string(imm_log_level level);

	/// @brief Turn the logger on or off
	///
	/// @param[in] enable A boolean describing if you'd like the logger to be active
	void set_enabled(bool enable);

	/// @brief Set the log level
	///
	/// @param[in] level Current log level the logger passes to the log
	void set_log_level(imm_log_level level);

private:
	/// brief Construct a singleton logger object
	///
	imm_logger();

	/// @brief Pointer to the handler. 
	///
	/// The handler can point to an inherited custom handler
	imm_logger_handler* handler;

	/// @brief The log lock is used to ensure thread safety when using the logger
	///
	std::mutex _log_lock;

	/// @brief This boolean can be used to turn the logger on or off in real time
	///
	bool _enabled;

	/// @brief This keeps track of the current log level. Anything below this level will not be logged
	///
	imm_log_level _imm_log_level;
};

#ifndef IMM_LOGGER
#define IMM_LOGGER imm_logger::get_logger()
#endif

==> Immersitech-ClearVoice-SDK-v2.0.1-MacOS.tar.gz: Immersitech-ClearVoice-SDK-v2.0.1-MacOS/include/immersitech_general.h <==
#pragma once

#ifdef HAVE_VISIBILITY
#if HAVE_VISIBILITY
#define IMMERSITECH_LOGGER_API __attribute__((__visibility__("default")))
#else
#define IMMERSITECH_LOGGER_API
#endif
#elif defined _WIN32
#define IMMERSITECH_LOGGER_API __declspec(dllexport)
#else
#define IMMERSITECH_LOGGER_API
#endif

#ifdef __ANDROID__
#include <android/log.h>
#define  LOG_TAG    "immersitech"
#define  LOGD(...)  __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__)
#endif

#include <stdbool.h>

/// @brief All error codes that may be produced by the library
///
typedef enum imm_error_code {
	IMM_ERROR_START = -10001,				///< This value allows you to identify the start of the enum
	IMM_ERROR_NONE,							///< This error code indicates the function worked successfully without error
	IMM_ERROR_HANDLE_NULL,					///< This error code indicates you have not initialized your imm_handle properly yet.
	IMM_ERROR_DATA_NULL,					///< This error code indicates you tried to use a buffer or variable that wasn't allocated properly.
	IMM_ERROR_DATA_LENGTH,					///< This error code indicates there is a mismatch between the size of the data buffer you provided and the size required.
	IMM_ERROR_NUM_CHANNELS,					///< This error code indicates that you tried to specify a number of channels outside the allowed bounds. Currently only 1 or 2 channels are allowed.
	IMM_ERROR_SAMPLE_RATE,					///< This error code indicates that you tried to specify an output sampling rate that is unsupported or an input sampling rate that doesn't work with the output sampling rate.
	IMM_ERROR_SPATIAL_QUALITY,				///< This error code indicates that you specified an invalid spatial quality value. Allowable values are from 1 to 5.
	IMM_ERROR_INVALID_ROOM_ID,				///< This error code indicates that you requested a room with an ID that doesn't exist or tried to create a room with an ID that already existed.
	IMM_ERROR_INVALID_PARTICIPANT_ID,		///< This error code indicates that you requested a participant with an ID that doesn't exist or tried to add a participant with an ID that already existed.
	IMM_ERROR_INVALID_CONTROL,				///< This error code indicates the control specified is not a valid one with the bounds of imm_audio_control
	IMM_ERROR_INVALID_VALUE,				///< This error code indicates the value provided is outside the bounds specified for the various imm_audio_control. It can also indicate you provided an invalid function parameter in general.
	IMM_ERROR_INVALID_HEADING,				///< This error code indicates you provide a heading that was outside the allowed values. Please reference imm_heading for allowed values.
	IMM_ERROR_INVALID_JSON_FILE,            ///< This error code indicates a syntax error in the provided JSON file (parameter, formating).
	IMM_ERROR_NO_INPUT_AUDIO,				///< This error code indicates that there was no input audio to be processed, meaning an output buffer could not be generated. Instead the output buffer will be zeroed.
	IMM_ERROR_PARTICIPANT_TYPE,				///< This error code matches the exceptions specified under imm_participant_type.
	IMM_ERROR_LIBRARY_NOT_YET_INITIALIZED,	///< This error code will be generated if you call any functions before initializing the library.
	IMM_ERROR_LICENSE_DATE_EXPIRED,			///< This error code indicates your license files no longer matches the date requirements. All audio effects will be bypassed.
	IMM_ERROR_LICENSE_VERSION_MISMATCH,		///< This error code indicates your license files version requirements no longer match. All audio effects will be bypassed.
	IMM_ERROR_LICENSE_TAMPERED,				///< This error code indicates your license file has been corrupted. All audio effects will be bypassed.
	IMM_ERROR_LICENSE_MISSING,				///< This error code indicates the path to your license file is invalid. All audio effects will be bypassed.
	IMM_ERROR_FILE_DOES_NOT_EXIST,          ///< This error code indicates that an invalid file path was provided (invalid path or invalid filename).
	IMM_ERROR_MISSING_OR_INVALID_PARAMETER, ///< This error code indicates that a parameter is either missing or invalid.
	IMM_ERROR_INVALID_FUNCTION_CALL,        ///< This error code indicates you called a function that is incompatible with your configuration, for example calling imm_cv_add_stream() while multistream_mode is disabled.
    IMM_ERROR_INVALID_STREAM_ID,            ///< This error code indicates that you supplied an invalid stream ID. Perhaps you've already removed this stream?
    IMM_ERROR_END							///< This value allows you to identify the end of the enum
}imm_error_code;

/// @brief Log level definition.
///
/// A message written to the log can have one of these levels of severity
typedef enum {
	IMM_LOG_VERBOSE,	///< This level is used for anything, no matter how unimportant
	IMM_LOG_DEBUG,		///< This level should be used only during a debugging session. It should print information that is helpful to the debugging process 
	IMM_LOG_INFO,		///< This level can be used to print generic information. For example, a participant was added to a room
	IMM_LOG_WARNING,	///< This level should be used for something that happened that could lead to an error or unexpected behavior
	IMM_LOG_ERROR		///< This level is exclusively for critical errors that have affected functionality
} imm_log_level;

#ifdef __cplusplus
extern "C" {
#endif

/// @brief Function to return the current version of the immersitech library
///
/// @return A string containing the version information for this instance of the Immersitech Library
IMMERSITECH_LOGGER_API const char* imm_get_version();

/// @brief Function to turn on or off logging within the Immersitech Library
/// 
/// If you disable logging by sending false to this function, nothing will ever be logged. No messages will be printed.
/// If you enable logging by sending true to this function. The Immersitech Library will log all messages equal to or above the set logging level.
/// By default, Immersitech Library will log all messages to the standard output.
/// By default, Immersitech Library will set the log level to Warnings and above only.
/// @param[in] enable This bool dictates whether you'd like to enable logging or not
/// @return An imm_error_code value specifying whether or not the call was a success
IMMERSITECH_LOGGER_API imm_error_code imm_enable_logging(bool enable);

/// @brief Function to set the log level of the Immersitech logger
/// 
/// The logger will only log messages that are equal to or greater than the currently set log level.
/// @param[in] level The log level you'd like to set. check the imm_log_level enum for details on which integer value corresponds to which log level
/// @return An imm_error_code value specifying whether or not the call was a success
IMMERSITECH_LOGGER_API imm_error_code imm_set_log_level(int level);

/// @brief Get a human readable string for an error code
///
/// This function allows you to print an error code enum value as its human readable equivalent by returning a string
/// @param[in] error_code The error code you'd like to get human readable form for
/// @return A string representing the corresponding error code enum value
IMMERSITECH_LOGGER_API const char* imm_error_code_to_string(imm_error_code error_code);

#ifdef __cplusplus
}
#endif
